'''
  Copyright (C) 2013 by Christopher Cooper, Lorena Barba

  Permission is hereby granted, free of charge, to any person obtaining a copy
  of this software and associated documentation files (the "Software"), to deal
  in the Software without restriction, including without limitation the rights
  to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
  copies of the Software, and to permit persons to whom the Software is
  furnished to do so, subject to the following conditions:

  The above copyright notice and this permission notice shall be included in
  all copies or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
  AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
  THE SOFTWARE.
'''
import matplotlib as mpl
mpl.use('Agg')
import matplotlib.pyplot as plt
import os
import numpy
import sys
import math
sys.path.append('../util')
from an_solution import *

def scanOutput(filename):
    
    flag = 0
    for line in file(filename):
        line = line.split()
        if len(line)>0:
            if line[0]=='Converged':
                iterations = int(line[2])
            if line[0]=='Total' and line[1]=='elements':
                N = int(line[-1])
            if line[0]=='Totals:':
                flag = 1
            if line[0]=='Esolv' and flag==1:
                Esolv = float(line[2])
            if line[0]=='Esurf' and flag==1:
                Esurf = float(line[2])
            if line[0]=='Ecoul' and flag==1:
                Ecoul = float(line[2])
            if line[0]=='Time' and flag==1:
                Time = float(line[2])

    return N, iterations, Esolv, Esurf, Ecoul, Time
            

mesh = numpy.array(['500','2K','8K','32K','130K'])

comm = './main.py PyGBeOrientation_sphere/inputs/sphere_fine.param PyGBeOrientation_sphere/inputs/molecule_dirichlet_'
out = 'output_aux'

print 'Runs for molecule + set phi/dphi surface'
N = numpy.zeros(len(mesh))
iterations = numpy.zeros(len(mesh))
Esolv = numpy.zeros(len(mesh))
Esurf = numpy.zeros(len(mesh))
Ecoul = numpy.zeros(len(mesh))
Time = numpy.zeros(len(mesh))
for i in range(len(mesh)):
    print 'Start run for mesh '+mesh[i]
    cmd = comm + mesh[i] + '.config > ' + out
    os.system(cmd)
    print 'Scan output file'
    N[i],iterations[i],Esolv[i],Esurf[i],Ecoul[i],Time[i] = scanOutput(out)
    

print 'Runs for isolated molecule'

comm = './main.py PyGBeOrientation_sphere/inputs/sphere_fine.param PyGBeOrientation_sphere/inputs/molecule_single_center_'
out = 'output_aux'

N_mol = numpy.zeros(len(mesh))
iterations_mol = numpy.zeros(len(mesh))
Esolv_mol = numpy.zeros(len(mesh))
Esurf_mol = numpy.zeros(len(mesh))
Ecoul_mol = numpy.zeros(len(mesh))
Time_mol = numpy.zeros(len(mesh))
for i in range(len(mesh)):
    print 'Start run for mesh '+mesh[i]
    cmd = comm + mesh[i] + '.config > ' + out
    os.system(cmd)
    print 'Scan output file'
    N_mol[i],iterations_mol[i],Esolv_mol[i],Esurf_mol[i],Ecoul_mol[i],Time_mol[i] = scanOutput(out)


print 'Runs for isolated surface'

comm = './main.py PyGBeOrientation_sphere/inputs/sphere_fine.param PyGBeOrientation_sphere/inputs/dirichlet_surface'
out = 'output_aux'

N_surf = numpy.zeros(len(mesh))
iterations_surf = numpy.zeros(len(mesh))
Esolv_surf = numpy.zeros(len(mesh))
Esurf_surf = numpy.zeros(len(mesh))
Ecoul_surf = numpy.zeros(len(mesh))
Time_surf = numpy.zeros(len(mesh))
for i in range(len(mesh)):
    print 'Start run for mesh '+mesh[i]
    cmd = comm + mesh[i] + '.config > ' + out
    os.system(cmd)
    print 'Scan output file'
    N_surf[i],iterations_surf[i],Esolv_surf[i],Esurf_surf[i],Ecoul_surf[i],Time_surf[i] = scanOutput(out)

Einter = Esolv + Esurf + Ecoul - Esolv_surf - Esurf_mol - Ecoul_mol - Esolv_mol - Esurf_surf - Ecoul_surf
total_time = Time+Time_mol+Time_surf

analytical = molecule_constant_potential(1., 1., 5., 4., 12., 0.125, 4., 80.)  

error = abs(Einter-analytical)/abs(analytical)

print '\nNumber of elements : '+str(N)
print 'Number of iteration: '+str(iterations)
print 'Interaction energy : '+str(Einter)
print 'Analytical solution: %f kcal/mol'%analytical
print 'Error              : '+str(error)
print 'Total time         : '+str(total_time)


flag = 0
for i in range(len(error)-1):
    rate = error[i]/error[i+1]
    if abs(rate-4)>0.6:
        flag = 1
        print 'Bad convergence for mesh %i to %i, with rate %f'%(i,i+1,rate)

if flag==0:
    print '\nPassed convergence test!'

Area = pi*(5**2+4**2)
N /= Area

font = {'family':'serif','size':10}
plt.rc('font',**font)
fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
asymp = N[0]*error[0]/N
ax.loglog(N, error, c='k', marker='o',ls=' ', mfc='w', ms=5, label='')
ax.loglog(N, asymp, c='k', marker='None', ls=':', lw=0.8, label=None)
loc = 1e5/Area
tex_loc = array((loc,N[0]*error[0]/7/Area))
#tex_angle = math.atan2(log(abs(asymp[-1]-asymp[0])),log(abs(N[-1]-N[0])))*180/math.pi
tex_angle = -40
ax.text(tex_loc[0], tex_loc[1],r'N$^{-1}$',fontsize=8,rotation=tex_angle)#,rotation_mode='anchor')
ax.set_ylabel('Relative error', fontsize=10)
ax.set_xlabel('Mesh density', fontsize=10)
fig.subplots_adjust(left=0.185, bottom=0.21, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_sphere/error_energy_molecule_dirichlet.pdf',dpi=80,format='pdf')
fig.clf()

fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.loglog(N, total_time, c='k', marker='o',ls=' ', mfc='w', ms=5, label='')
ax.loglog(N, N*log10(N)*total_time[2]/(N[2]*log10(N[2])), c='k', marker='None', ls=':', lw=0.8, label=None)
ax.text(1e5/Area, 7.0e2*log10(1e5/Area)*total_time[0]/(N[0]*log10(N[0])),'NlogN',fontsize=8,rotation=47)
ax.set_ylabel('Total time [s]', fontsize=10)
ax.set_xlabel('Mesh density', fontsize=10)
ax.set_ylim([total_time[0]/1.2, total_time[-1]*1.2])
fig.subplots_adjust(left=0.185, bottom=0.21, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_sphere/total_time.pdf',dpi=80,format='pdf')
fig.clf()

fig = plt.figure(figsize=(3,2), dpi=80)
ax = fig.add_subplot(111)
ax.semilogx(N, iterations, c='k', marker='o',ls=' ', mfc='w', ms=5, label='')
ax.set_ylabel('Iterations', fontsize=10)
ax.set_xlabel('Mesh density', fontsize=10)
ax.set_ylim([0,80])
fig.subplots_adjust(left=0.185, bottom=0.21, right=0.965, top=0.95)
fig.savefig('PyGBeOrientation_sphere/iterations.pdf',dpi=80,format='pdf')
fig.clf()
